# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

initlslinregfit <- function(xl, ql, rtl) {
    .Call(`_lsReg_initlslinregfit`, xl, ql, rtl)
}

lslinregfit <- function(y, xl, xr, beta0, ql, rtl, bt, bb, betat, betab, qr, rtr, rbr, h, t, zb) {
    .Call(`_lsReg_lslinregfit`, y, xl, xr, beta0, ql, rtl, bt, bb, betat, betab, qr, rtr, rbr, h, t, zb)
}

initlslinreglrt <- function(y, xl, beta, resids, loglike) {
    .Call(`_lsReg_initlslinreglrt`, y, xl, beta, resids, loglike)
}

lslinreglrt <- function(y, xl, xr, bt, bb, resids, loglike, testvalue) {
    .Call(`_lsReg_lslinreglrt`, y, xl, xr, bt, bb, resids, loglike, testvalue)
}

initlslinregscore <- function(y, xl, beta, resids0, s2, xtx, xtx0inv) {
    .Call(`_lsReg_initlslinregscore`, y, xl, beta, resids0, s2, xtx, xtx0inv)
}

lslinregscore <- function(xl, xr, xtx0inv, resids0, s2, xtx, info, score, testvalue) {
    .Call(`_lsReg_lslinregscore`, xl, xr, xtx0inv, resids0, s2, xtx, info, score, testvalue)
}

initlslinregrobustscore <- function(y, xl, beta, resids0, xlr0, s2, xtx, xtx0inv, uut, cmat) {
    .Call(`_lsReg_initlslinregrobustscore`, y, xl, beta, resids0, xlr0, s2, xtx, xtx0inv, uut, cmat)
}

lslinregrobustscore <- function(xl, xlr0, xr, xtx0inv, resids0, xrr0, uut, cmat, score, info, testvalue) {
    .Call(`_lsReg_lslinregrobustscore`, xl, xlr0, xr, xtx0inv, resids0, xrr0, uut, cmat, score, info, testvalue)
}

initlslinregwald <- function(rtl, rtlinv) {
    .Call(`_lsReg_initlslinregwald`, rtl, rtlinv)
}

lslinregwaldtest <- function(y, xl, xr, bt, bb, rtlinv, rtr, rbr, resids, s2, rtrinv, rbrinv, xtxinv, testvalue) {
    .Call(`_lsReg_lslinregwaldtest`, y, xl, xr, bt, bb, rtlinv, rtr, rbr, resids, s2, rtrinv, rbrinv, xtxinv, testvalue)
}

lslinregrobustwaldtest <- function(y, xl, xr, bt, bb, rtlinv, rtr, rbr, xlr, xrr, resids, s2, rtrinv, rbrinv, xtxinv, hwinfo, testvalue) {
    .Call(`_lsReg_lslinregrobustwaldtest`, y, xl, xr, bt, bb, rtlinv, rtr, rbr, xlr, xrr, resids, s2, rtrinv, rbrinv, xtxinv, hwinfo, testvalue)
}

initlslogregfit <- function(y, xl, beta0, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, w0inv, xlw0, k, abx, expabx, expabxp1, expitabx) {
    .Call(`_lsReg_initlslogregfit`, y, xl, beta0, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, w0inv, xlw0, k, abx, expabx, expabxp1, expitabx)
}

lslogregfit <- function(y, xl, xr, beta0, yp0, ql, rtl, k0, w0, winv0, beta, bt, bb, betat, betab, abx, expabx, expabxp1, expitabx, h, k, qr, rtr, rbr, scoret, scoreb, t, xrw, yp, zt, zb) {
    .Call(`_lsReg_lslogregfit`, y, xl, xr, beta0, yp0, ql, rtl, k0, w0, winv0, beta, bt, bb, betat, betab, abx, expabx, expabxp1, expitabx, h, k, qr, rtr, rbr, scoret, scoreb, t, xrw, yp, zt, zb)
}

lslogreglikelihood <- function(y, abx, expabxp1) {
    .Call(`_lsReg_lslogreglikelihood`, y, abx, expabxp1)
}

lslogreglrt <- function(y, abx, expabxp1, loglike, testvalue) {
    .Call(`_lsReg_lslogreglrt`, y, abx, expabxp1, loglike, testvalue)
}

initlslogregscore <- function(y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, expabxp1, expitabx, xlw0) {
    .Call(`_lsReg_initlslogregscore`, y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, expabxp1, expitabx, xlw0)
}

score_test <- function(xr, w0, xlw0, xtxinv0, resids0, xtx0, xrw0, info0, score0, testvalue) {
    .Call(`_lsReg_score_test`, xr, w0, xlw0, xtxinv0, resids0, xtx0, xrw0, info0, score0, testvalue)
}

initlslogregrobustscore <- function(y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, expabxp1, expitabx, xlw0, xlr0, uut) {
    .Call(`_lsReg_initlslogregrobustscore`, y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, expabxp1, expitabx, xlw0, xlr0, uut)
}

robust_score_test <- function(xr, resids0, w0, xlw0, xlr0, xtx0, xtx0inv, xrw0, xrr0, uut, cmat, robinfo0, robscore0, testvalue) {
    .Call(`_lsReg_robust_score_test`, xr, resids0, w0, xlw0, xlr0, xtx0, xtx0inv, xrw0, xrr0, uut, cmat, robinfo0, robscore0, testvalue)
}

log_wald_test <- function(xl, xr, betab, expitabx, w, xw, q, r, rinv, xtx, xtxinv, testvalue) {
    .Call(`_lsReg_log_wald_test`, xl, xr, betab, expitabx, w, xw, q, r, rinv, xtx, xtxinv, testvalue)
}

log_robustwald_test <- function(xl, xr, betab, expitabx, resids, w, xw, xrr, q, r, rinv, xtx, xtxinv, hws2, testvalue) {
    .Call(`_lsReg_log_robustwald_test`, xl, xr, betab, expitabx, resids, w, xw, xrr, q, r, rinv, xtx, xtxinv, hws2, testvalue)
}

initlspoisregfit <- function(y, xl, beta0, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, w0inv, xlw0, abx, expabx) {
    .Call(`_lsReg_initlspoisregfit`, y, xl, beta0, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, w0inv, xlw0, abx, expabx)
}

lspoisregfit <- function(y, xl, xr, beta0, yp0, ql, rtl, k0, w0, winv0, beta, bt, bb, betat, betab, abx, expabx, h, k, qr, rtr, rbr, scoret, scoreb, t, xrw, yp, zt, zb) {
    .Call(`_lsReg_lspoisregfit`, y, xl, xr, beta0, yp0, ql, rtl, k0, w0, winv0, beta, bt, bb, betat, betab, abx, expabx, h, k, qr, rtr, rbr, scoret, scoreb, t, xrw, yp, zt, zb)
}

lspoisreglikelihood <- function(y, abx, expabx, lnfacty) {
    .Call(`_lsReg_lspoisreglikelihood`, y, abx, expabx, lnfacty)
}

lspoisreglrt <- function(y, abx, expabx, lnfacty, loglike, testvalue) {
    .Call(`_lsReg_lspoisreglrt`, y, abx, expabx, lnfacty, loglike, testvalue)
}

initlspoisregscore <- function(y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, xlw0) {
    .Call(`_lsReg_initlspoisregscore`, y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, xlw0)
}

initlspoisregrobustscore <- function(y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, xlw0, xlr0, uut) {
    .Call(`_lsReg_initlspoisregrobustscore`, y, xl, beta, ql0, rtl0, rtl0inv, xtx0, xtx0inv, yp0, w0, abx, expabx, xlw0, xlr0, uut)
}

pois_wald_test <- function(xl, xr, betab, expabx, w, xw, q, r, rinv, xtx, xtxinv, testvalue) {
    .Call(`_lsReg_pois_wald_test`, xl, xr, betab, expabx, w, xw, q, r, rinv, xtx, xtxinv, testvalue)
}

pois_robustwald_test <- function(xl, xr, betab, expabx, resids, w, xw, xrr, q, r, rinv, xtx, xtxinv, hws2, testvalue) {
    .Call(`_lsReg_pois_robustwald_test`, xl, xr, betab, expabx, resids, w, xw, xrr, q, r, rinv, xtx, xtxinv, hws2, testvalue)
}

